using System;
using System.Collections.Generic;
using System.Text;
using OHE.LCP.Admin.Messages.Interface;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Web;
using Microsoft.Practices.ObjectBuilder;
using OHE.FoundationModules.Base.Business;
using OHE.FoundationModules.ProviderServices.BusinessEntities;
using fm = OHE.FoundationModules.CollegeInformation.Module;

namespace OHE.LCP.Admin.Messages.Services
{
    /// <summary>
    /// The Message Service that is used by the controller
    /// </summary>
    public class MessageService : IMessageService
    {
        //Private variable for accessing the message repository
        private MessageRepository _messageRepository = new MessageRepository("dbConn");
        //Private variable for accessing the message target repository
        private MessageTargetRepository _messageTargetRepository = new MessageTargetRepository("dbConn");
        //Private variable for accessing the message read repository
        private MessageReadRepository _messageReadRepository = new MessageReadRepository("dbConn");
        //Private variable for the Provider Service Manager
        fm.IProviderServiceManager _providerServiceManager = null;        

        #region IMessageService Members
        
        [ServiceDependency]
        public fm.IProviderServiceManager ProviderServiceManager
        {
            set 
            {
                _providerServiceManager = value;
            }
        }
        
        /// <summary>
        /// Gets all of the messages in the system.
        /// </summary>
        /// <returns></returns>
        public IList<Message> GetAllMessages()
        {
            return _messageRepository.GetAllFromMessage();
        }

        /// <summary>
        /// Gets an individual message.  Also populates the message with the correct message targets and message reads
        /// </summary>
        /// <param name="messageID">The ID of the message</param>
        /// <returns></returns>
        public Message GetMessage(int messageID)
        {
            Message _message = _messageRepository.GetMessageByMessageID(messageID);
            _message.MessageTargets = _messageTargetRepository.GetMessageTargetByMessageID(messageID);
            _message.MessageReads = _messageReadRepository.GetMessageReadByMessageID(messageID);
            return _message;
        }

        /// <summary>
        /// Marks a message as read by a particular user.
        /// </summary>
        /// <param name="messageID">The ID of the message that was read</param>
        /// <param name="LCPUserID">The ID of the LCP user that read the message</param>
        /// <param name="ReadDateTime">The date and time that the message was read</param>
        public void ReadMessage(int messageID, string LCPUserID, DateTime ReadDateTime)
        {
            //First check and make sure that there isn't already a MessageRead for this particular user and message
            List<MessageRead> _messageReads = _messageReadRepository.GetMessageReadByMessageID(messageID);
            bool _found = false;
            foreach (MessageRead _mr in _messageReads)
            {
                if(_mr.LCPUserID == LCPUserID)
                {
                    _found = true;
                    break;
                }
            }

            if (!_found)
            {
                //Now create a Message Read object and add
                MessageRead _messageRead = new MessageRead(LCPUserID, messageID, 0, ReadDateTime);
                _messageReadRepository.Add(_messageRead);
            }
        }

        /// <summary>
        /// Gets the list of messages for an LCP user
        /// </summary>
        /// <param name="RoleIDs">A comma delimited list of RoleIDs for the user</param>
        /// <param name="SchoolIDs">A comma delimited list of SchoolIDs for the user</param>
        /// <param name="ProgramIDs">A comma delimited list of ProgramIDs for the user</param>
        /// <param name="ConcentrationIDs">A comma delimited list of ConcentrationsIDs for the user</param>
        /// <param name="CourseIDs">A comma delimited list of CourseIDs for the user</param>
        /// <param name="SectionIDs">A comma delimited list of SectionIDs for the user</param>
        /// <param name="PersonID">The ID of the LCP user</param>
        /// <returns></returns>
        public List<Message> GetMessagesForLCP(string RoleIDs, string SchoolIDs, string ProgramIDs, string ConcentrationIDs, string CourseIDs, string SectionIDs, string PersonID)
        {
            List<Message> _messages = _messageRepository.GetMessagesForLCP(ConcentrationIDs, CourseIDs, PersonID, ProgramIDs, RoleIDs, SchoolIDs, SectionIDs);
            foreach(Message _message in _messages)
            {
                _message.MessageReads = _messageReadRepository.GetMessageReadByMessageID(_message.MessageID);
            }
            return _messages; 
        }

        /// <summary>
        /// Adds a new message to the repository.
        /// </summary>
        /// <param name="message">The message to be added.</param>
        public void InsertMessage(Message message)
        {
            //First we need to add the message, then we need to add the individual message targets
            _messageRepository.Add(message);
            foreach (MessageTarget _messageTarget in message.MessageTargets)
            {
                _messageTarget.MessageID = message.MessageID;
                _messageTargetRepository.Add(_messageTarget);
            }
        }

        /// <summary>
        /// Updates a message in the repository.
        /// </summary>
        /// <param name="message">The message to be updated</param>
        public void UpdateMessage(Message message)
        {
            //First we need to delete all of the old targets from the database
            List<MessageTarget> _oldMessageTargets = _messageTargetRepository.GetMessageTargetByMessageID(message.MessageID);
            _messageTargetRepository.Remove(_oldMessageTargets);
            //Save the message
            _messageRepository.Save(message);
            //Now add in all of the new targets
            foreach (MessageTarget _messageTarget in message.MessageTargets)
            {
                _messageTarget.MessageID = message.MessageID;
                _messageTargetRepository.Add(_messageTarget);
            }
        }

        /// <summary>
        /// Deletes a message and all of its associated reads and targets
        /// </summary>
        /// <param name="messageID">The ID of the message to be deleted</param>
        public void DeleteMessage(int messageID)
        {
            Message _message = this.GetMessage(messageID);
            _messageReadRepository.Remove(_message.MessageReads);
            _messageTargetRepository.Remove(_message.MessageTargets);
            _messageRepository.Remove(messageID);
        }

        //public void DeleteMessageTarget(int messageTargetID)
        //{
        //    _messageTargetRepository.Remove(messageTargetID);
        //}

        //public void DeleteMessageTarget(List<MessageTarget> messageTargets)
        //{
        //    foreach (MessageTarget _target in messageTargets)
        //    {
        //        this.DeleteMessageTarget(_target.MessageTargetID);
        //    }
        //}

        //public void InsertMessageTarget(MessageTarget messageTarget)
        //{
        //    _messageTargetRepository.Add(messageTarget);
        //}
        
        /// <summary>
        /// Gets the dictionary of business entities for all of the Colleges, Programs and Concentrations in the system.
        /// </summary>
        /// <param name="structureType">The structure type for the business entities that come back.</param>
        /// <returns></returns>
        public Dictionary<string, List<IBusinessEntity>> GetCollegeProgramConcentrations(Enums.BusinessEntityStructureType structureType)
        {
            return _providerServiceManager.GetCollegeProgramConcentrations(structureType);
        }
        
        #endregion
    }
}
